/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.energy;

/**
 * This listener gives {@link EnergyComponent}s access to the {@link EnergyModel}.
 * 
 * @author Julian Zobel
 * @version 1.0, 07.09.2018
 */
public interface EnergyEventListener {
	
	/**
	 * Notify the energy model that a component has consumed energy.
	 * 
	 * @param component The energy component that has consumed energy.
	 * @param consumedEnergy The amount of energy that was consumed by that component (uJ)
	 */
	public void componentConsumedEnergy(EnergyComponent component, double consumedEnergy);

	/**
	 * Allows a component to check, if it is able to be turned on.
	 * 
	 * @param component The energy component that is checked to be able to go online.
	 * @return False if there is any reason, that the component cannot be turned on; true if it can be turned on.
	 */
	public boolean componentCanBeActivated(EnergyComponent component);
}
