/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.topology.component;

import java.util.LinkedList;

import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.uavsupport.callbacks.ReachedLocationCallback;

/**
 * A controllable location actuator, that can be turned on and off, gives access
 * to the current location and battery levels, and also receives a location for
 * a target that the actuator will then approach.
 * 
 * @author Julian Zobel
 *
 */
public interface ControllableLocationActuator {

	public boolean isActive();

	public boolean activate();

	public boolean deactivate();

	public PositionVector getCurrentLocation();

	public double getCurrentBatteryLevel();

	public LinkedList<PositionVector> getTargetLocations();
	
	public void setTargetLocation(PositionVector targetLocation, ReachedLocationCallback cb);

	public void addTargetLocation(PositionVector targetLocation, ReachedLocationCallback cb);

	public void setTargetLocationRoute(LinkedList<PositionVector> route, ReachedLocationCallback cb);

	public void removeAllTargetLocations();

}
