/*
 * Copyright (c) 2005-2010 KOM � Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.configs;

import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.energy.EnergyConfiguration;
import de.tud.kom.p2psim.impl.energy.components.ActuatorEnergyComponent;

/**
 * Energy Configuration for {@link ActuatorEnergyComponent}s.
 * 
 * @author Julian Zobel
 * @version 1.0, 11.09.2018
 */
public class ActuatorEnergyConsumptionConfig implements EnergyConfiguration<ActuatorEnergyComponent> {

	private int numberOfActuators;
	private double volt; 	
	private double hoverAmp; // in ampere
	private double flyAmp; // in ampere
	
	@Override
	public ActuatorEnergyComponent getConfiguredEnergyComponent(SimHost host) {
		return new ActuatorEnergyComponent(numberOfActuators, volt, hoverAmp, flyAmp);		
	}

	@Override
	public String getHelp() {		
		return "Fix actuator energy consumption config";
	}

	@Override
	public boolean isWellConfigured() {
		if(numberOfActuators >= 1 && volt > 0
				&& hoverAmp > 0	&& flyAmp > 0)
			return true;
					
		return false;
	}

	public void setNumberOfActuators(int num) {
		numberOfActuators = num;
	}
	
	public void setHoverAmp(double ampereHovering) {
		this.hoverAmp = ampereHovering;
	}

	public void setFlyAmp(double ampereFlying) {
		this.flyAmp = ampereFlying;
	}
	
	public void setVolt(double voltage) {
		this.volt = voltage;
	}
	

}
