/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.models;

import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.energy.Battery;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyEventListener;
import de.tud.kom.p2psim.api.network.SimNetInterface;

/**
 * Energy Model based on multiple exchangeable components. Each component states the amount of consumed energy,
 *  which the energy model than will remove from the battery.
 * 
 * @author Julian Zobel
 * @version 1.0, 11.09.2018
 */
public class ComponentBasedEnergyModel extends AbstractEnergyModel implements EnergyEventListener {

	public ComponentBasedEnergyModel(SimHost host, Battery bat) {
		super(host, bat);
		
	}

	@Override
	public void registerComponent(EnergyComponent comp) {
		energyComponents.add(comp);
		comp.setEnergyEventListener(this);
	}

	@Override
	public void componentConsumedEnergy(EnergyComponent component, double consumedEnergy) {
		
		if (!bat.isEmpty()) {
			
			bat.consumeEnergy(consumedEnergy);
					
			monitorEnergyConsumation(component, consumedEnergy);
			
			if (bat.isEmpty()) {
				
				monitorEmptyBattery();
				
				/*
				 * Battery is now empty. Go offline.
				 */				
				for (SimNetInterface net : getHost().getNetworkComponent()
						.getSimNetworkInterfaces()) {
					net.goOffline();
				}
			}
		}
	}

	@Override
	public boolean componentCanBeActivated(EnergyComponent component) {
		if(!bat.isEmpty()) {
			return true;
		}
		
		return false;
	}
}
