/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.aerial;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.topology.movement.MovementModel;
import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tud.kom.p2psim.api.topology.movement.SimUAVLocationActuator;
import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.EventHandler;
import de.tudarmstadt.maki.simonstrator.api.Time;

/**
 * Movement model container for all UAVs. This primarily triggers the local movement models of the respective UAVs. 
 * 
 * @author Julian Zobel
 * @version 1.0, 6 Sep 2018
 */
public class UAVMovement implements MovementModel, EventHandler {

	private List<SimUAVLocationActuator> actuators = new LinkedList<>();
	protected long timeBetweenMoveOperation = Time.SECOND;
	
	public UAVMovement() {
		Event.scheduleImmediately(this, null, 0);
	}
	
	@Override
	public void addComponent(SimLocationActuator actuator) {
		if(!actuators.contains(actuator) && actuator instanceof SimUAVLocationActuator)
			actuators.add((SimUAVLocationActuator) actuator);		
	}

	@Override
	public void placeComponent(SimLocationActuator actuator) {
		//
	}

	@Override
	public void setTimeBetweenMoveOperations(long time) {
		this.timeBetweenMoveOperation = time;		
	}

	@Override
	public void eventOccurred(Object content, int type) {
		this.triggerComponentMovement();		
	}
	
	private void triggerComponentMovement() {
		for (SimUAVLocationActuator actuator : actuators) {
			actuator.getUAVMovement().move(timeBetweenMoveOperation);
		}
		
		Event.scheduleWithDelay(timeBetweenMoveOperation, this, null, 1);
	}

	
}
