/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.local;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.route.Route.RouteSegment;

/**
 * Implementation of a {@link RouteSegment}
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, Aug 10, 2017
 */
public class RouteSegmentImpl implements RouteSegment {

	private final String id;

	private final int fromRouteIndex;

	private final List<Location> routeLocations;

	/**
	 * The segment and associated points (via their index)
	 * 
	 * @param id
	 * @param fromRouteIndex
	 */
	public RouteSegmentImpl(String id, int fromRouteIndex,
			Location startLocation) {
		this.id = id;
		this.fromRouteIndex = fromRouteIndex;
		this.routeLocations = new LinkedList<>();
		this.routeLocations.add(startLocation);
	}

	public void addRouteLocation(Location location) {
		this.routeLocations.add(location);
	}

	@Override
	public String getSegmentId() {
		return id;
	}

	@Override
	public List<Location> getSegmentLocations() {
		return Collections.unmodifiableList(routeLocations);
	}

	public int getFromRouteIndex() {
		return fromRouteIndex;
	}

}
