/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm;

import java.util.LinkedHashSet;
import java.util.Set;

import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.route.Route;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.route.RouteSensor;

/**
 * The OSM-based models provide an implementation of the {@link RouteSensor},
 * which is added as a new host component. It is created by supporting mobility
 * models, registers itself with the host container and can then be retrieved
 * via the host container from w.g., a local movement strategy.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, Aug 10, 2017
 */
public class RouteSensorComponent implements RouteSensor {

	private final SimLocationActuator actuator;

	private Route currentRoute;

	private final Set<RouteListener> listeners;

	private final Set<RouteSegmentListener> segmentListeners;

	public RouteSensorComponent(SimLocationActuator actuator) {
		this.actuator = actuator;
		this.listeners = new LinkedHashSet<>();
		this.segmentListeners = new LinkedHashSet<>();
		actuator.getHost().registerComponent(RouteSensorComponent.this);
	}

	@Override
	public void initialize() {
		// 
	}

	@Override
	public void shutdown() {
		// 
	}

	public SimLocationActuator getActuator() {
		return actuator;
	}

	@Override
	public Host getHost() {
		return actuator.getHost();
	}

	@Override
	public Route getRoute() {
		return currentRoute;
	}

	@Override
	public void addRouteListener(RouteListener listener) {
		listeners.add(listener);
	}

	@Override
	public void removeRouteListener(RouteListener listener) {
		listeners.remove(listener);
	}

	@Override
	public void addRouteSegmentListener(RouteSegmentListener listener) {
		segmentListeners.add(listener);
	}

	@Override
	public void removeRouteSegmentListener(RouteSegmentListener listener) {
		segmentListeners.remove(listener);
	}

	/*
	 * The following methods are used by the mobility models.
	 */

	public void setNewRoute(Route newRoute) {
		// notifies listeners
		listeners.forEach(l -> l.onChangedRoute(this, currentRoute, newRoute));
		currentRoute = newRoute;
	}

	public void reachedDestination() {
		// notifies listeners
		listeners.forEach(l -> l.onReachedDestination(this, currentRoute));
	}

	public Set<RouteSegmentListener> getSegmentListeners() {
		return segmentListeners;
	}

}
