/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.views.fiveg.models;
/**
 * A cut off model based on the heaviside step function.
 * 
 * https://de.wikipedia.org/wiki/Heaviside-Funktion
 * 
 * cut(u) = a * θ(c * u + d) + b
 * @author Marc Schiller
 * @version 1.0, 15 Dec 2016
 */
public class CutOffModel extends AbstractModel {
	
	private double a = 1;

	private double b = 0;

	private double c = 1;

	private double d = 0;
	
	public void setA(double a) {
		this.a = a;
	}

	public void setB(double b) {
		this.b = b;
	}

	public void setC(double c) {
		this.c = c;
	}

	public void setD(double d) {
		this.d = d;
	}
	
	private double heaviside(double x) {
		if(x < 0) {
			return 0;
		} else {
			return 1;
		}
	}

	@Override
	public long getLong(int users) {
		return (long) getDouble(users);
	}

	@Override
	public double getDouble(int users) {
		return this.a * heaviside(this.c * users + this.d) + this.b;
	}
	
	@Override
	public String toString() {
		return "CutOffModel: cut(u) = " + this.a + " * θ(" + this.c + " * u + " + this.d + ") + " + this.b;
	}

}
