/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.vehicular.caching.decision;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.NumericVectoralProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.jam.VectoralJamProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision.CacheDecisionStrategy;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.RoadInformation;

public class AveragingCacheDecisionStrategy implements CacheDecisionStrategy {

	@Override
	public <T extends PointInformation> T decideOnCorrectInformation(
			List<T> pSimilarPointInformation) {
		if (pSimilarPointInformation.size() == 1) {
			T decision = pSimilarPointInformation.get(0);
			return decision;
		} else if (pSimilarPointInformation.size() == 0) {
			return null;
		}
		
		double sum = 0;
		double count = 0;
		
		NumericVectoralProperty cloned = null;
		
		for (T t : pSimilarPointInformation) {
			RoadInformation roadInformation = ((RoadInformation) t);
			
			NumericVectoralProperty property = (NumericVectoralProperty) roadInformation.getValue();
			if (cloned == null) {
				cloned = property.clone();
			}
			
			sum += property.getMostProbableValue();
			count++;
		}

		double value = sum / count;
		
		if (cloned instanceof VectoralJamProperty) {
			((VectoralJamProperty) cloned).setSpeed(((int)(value / VectoralJamProperty.SCALING)) * VectoralJamProperty.SCALING);
		} else {
			throw new AssertionError("Unknown data type " + cloned.getClass().getSimpleName());
		}

		return (T) new RoadInformation(cloned);
	}

}
