/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.vehicular.caching.decision;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision.CacheDecisionStrategy;

public enum CacheDecisionStrategyType {
	DEFAULT(NewestCacheDecisionStrategy.class), NEWEST(NewestCacheDecisionStrategy.class), MAJORITY(MajorityVotingCacheDecisionStrategy.class), AVERAGING(AveragingCacheDecisionStrategy.class), TTL(TTLbasedCacheDecisionStrategy.class), OPTIMAL(OptimalCacheDecisionStrategy.class), RANDOM(RandomCacheDecisionStrategy.class), TTL_VECTOR(TTLbasedVectoralCacheDecisionStrategy.class), MAJORITY_VECTOR(MajorityVotingVectoralCacheDecisionStrategy.class);

	private final Class<? extends CacheDecisionStrategy> decisionStrategy;
	private final Map<String, String> params = new HashMap<>();

	private CacheDecisionStrategyType(final Class<? extends CacheDecisionStrategy> decisionStrategy) {
		this.decisionStrategy = decisionStrategy;
	}

	public CacheDecisionStrategy getDecisionStrategy() {
		try {
			try {
				Constructor<? extends CacheDecisionStrategy> constructor = decisionStrategy.getConstructor(Map.class);
				return constructor.newInstance(params);
			} catch (NoSuchMethodException | SecurityException | IllegalArgumentException | InvocationTargetException e) {
				return decisionStrategy.newInstance();
			}
		} catch (InstantiationException | IllegalAccessException e) {
			throw new AssertionError(e);
		}
	}

	public void addAttribute(String pKey, String pValue) {
		params.put(pKey, pValue);
	}
}
