/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.vehicular.caching.decision;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision.CacheDecisionStrategy;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;

public class MajorityVotingCacheDecisionStrategy implements CacheDecisionStrategy {

	@Override
	public <T extends PointInformation> T decideOnCorrectInformation(
			List<T> pSimilarPointInformation) {
		Map<Object, Integer> voting = new HashMap<>();
		for (T t : pSimilarPointInformation) {
			if (!voting.containsKey(t.getValue())) {
				voting.put(t.getValue(), 0);
			}

			voting.put(t.getValue(), voting.get(t.getValue()) + 1);
		}

		Entry<Object, Integer> maxEntry = null;
		for (Entry<Object, Integer> entry : voting.entrySet()) {
			if (maxEntry == null) {
				maxEntry = entry;
			}

			if (maxEntry.getValue() < entry.getValue()) {
				maxEntry = entry;
			}
		}

		long maxTimestamp = 0;
		T maxFitting = null;
		for (T t : pSimilarPointInformation) {
			long timestamp = t.getDetectionDate();

			if (t.getValue().equals(maxEntry.getKey()) && timestamp > maxTimestamp) {
				maxTimestamp = timestamp;
				maxFitting = t;
			}
		}

		return maxFitting;
	}

}
