/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.vehicular.caching.decision;

import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.VectoralProperty;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.data.jam.JamProperty;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.caching.decision.CacheDecisionStrategy;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.AvailableInformationAttributes;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.information.PointInformation;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;

public class OptimalCacheDecisionStrategy implements CacheDecisionStrategy {
	public OptimalCacheDecisionStrategy() {
	}

	@Override
	public <T extends PointInformation> T decideOnCorrectInformation(
			List<T> pSimilarPointInformation) {
		if (pSimilarPointInformation.size() == 1) {
			T decision = pSimilarPointInformation.get(0);
			return decision;
		} else if (pSimilarPointInformation.size() == 0) {
			return null;
		}
		
		RoadNetworkEdge edge = (RoadNetworkEdge) pSimilarPointInformation.get(0).getAttribute(AvailableInformationAttributes.EDGE);
		
		double actualSpeed = edge.getCurrentSpeed();
		
		JamProperty jamProperty = edge.getJamProperty();

		long maxTimestamp;
		if (jamProperty != null) {
			maxTimestamp = jamProperty.getDetectionDate();
		} else {
			maxTimestamp = -1;
		}

		double difference = Double.MAX_VALUE;
		T maxFitting = null;
		for (T t : pSimilarPointInformation) {
			long timestamp = t.getDetectionDate();

			Object currentValue = t.getValue();
			if (currentValue instanceof VectoralProperty) {
				currentValue = ((VectoralProperty) currentValue).getMostProbableValue();
			}
			
			if (timestamp >= maxTimestamp) {
				if (currentValue.equals(actualSpeed)) {
					maxFitting = t;
					difference = 0;
				} else if (currentValue instanceof Number && Math.abs(((Number)currentValue).doubleValue() - actualSpeed) < difference) {
					maxFitting = t;
					difference = Math.abs(((Number)currentValue).doubleValue() - actualSpeed);
				}
			}
		}

		if (maxFitting == null) {
			maxTimestamp = -1;

			for (T t : pSimilarPointInformation) {
				long timestamp = t.getDetectionDate();

				if (timestamp >= maxTimestamp) {
					maxTimestamp = timestamp;
					maxFitting = t;
				}
			}
		}

		return maxFitting;
	}
}
